/*! Copyright (C) [2010] [David Mainzer]
 * 
 *  This program is free software; you can redistribute it and/or modify it under the terms 
 *  of the GNU General Public License as published by the Free Software Foundation; either 
 *  version 3 of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU General Public License for more details.
 *  
 *	You should have received a copy of the GNU General Public License along with this program; 
 *	if not, see <http://www.gnu.org/licenses/>.  
 */

import vrml.*;
import vrml.field.*;
import vrml.node.*;

public class ParticleSystem extends Script 
{
	private SFNode ParticlesIN;

	private int realNumParticles;							//!< real #particles

	private float tmpPendelBBox [] = new float[3];			//!< bbox (pendulum)
	private float tmpPendelBBoxCenter [] = new float[3];	//!< bbox center point (pendulum)
	
	
	private float energyPoint [] = new float[3];			//!< center point (sphere of the pendulum)

	// number of particles
	private int maxparticle;								//!< max #particles (read from VRML file)
	
	Particles [] myParticleArray;
	
	long runtime   = (long) 0.0;										//!< simulation run time (in ms)
	long starttime = (long) 0.0;										//!< simulation start time (in ms)
	
	/*! \brief initialization of the hole particle system
	 *  creates the particles (number is given by VRML file)
	 * 
	 */
	@Override
	public void initialize()
	{
		System.out.println("###############################");
		System.out.println("## ParticleSystem using JAVA ##");
		System.out.println("###############################");
		System.out.println("");
		System.out.println("Initializing java ... \n");
	
		// get number of particles
		maxparticle = ((vrml.field.SFInt32)getField("particles")).getValue();
		
		myParticleArray = new Particles[maxparticle];
		
		// get particle-node (MUST exist in the VRML file)
		ParticlesIN = (SFNode)getField("ParticlesIN");
		// get child of particle-node
    	MFNode nodeParticales = (MFNode)((Node)ParticlesIN.getValue()).getExposedField("children");
   	
    	System.out.print("Create particles ... " );
    	
    	for(int i=0; i<maxparticle; i++)
    	{
    		String codeVRML = "Transform { \n" +
								" #ID" + i +"\n" +
							    "    translation 0 0 0 \n" +
							    "    scale 1 1 1 \n" +
							    "    children \n" +
							    "        Shape { \n" +
							    "            appearance Appearance { \n" +
							    "                material Material { \n" +
							    "                    emissiveColor 1 0 0 \n" +
							    "                } \n" +
							    "            } \n" +
							    "            geometry Box { \n" +
							    "                size 0.01 0.01 0.01 \n" +
							    "            } \n" +
							    "        } \n" +
							    "} \n";

			vrml.Browser browser = getBrowser();
			BaseNode[] nodes = null;
			
			try
			{
				nodes = browser.createVrmlFromString(codeVRML);
			} catch (InvalidVRMLSyntaxException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			nodeParticales.addValue( nodes[0] );
			
			myParticleArray[i] = new Particles();
    	}

    	System.out.print("done\n");
	
   		realNumParticles = nodeParticales.getSize();

		System.out.println("Number of particles created: " + realNumParticles);
		
		// set start time
		starttime = System.currentTimeMillis();
	}

	/** set uptime of the simulation (in ms)
	 * 
	 */
	public void setTimer ()
	{
		long timeInMillis = System.currentTimeMillis();
		runtime = timeInMillis - starttime;
	}

	@Override
    public void processEvent(Event e) 
	{

		String name = e.getName();
		// store new pendulum center-point
		if( name.equals("bboxPendelCenter") )
		{
			ConstSFVec3f tmp_pendelCenter = (ConstSFVec3f)e.getValue();
			tmp_pendelCenter.getValue(tmpPendelBBoxCenter);
		}
		
		if( name.equals("bboxPendel") )
		{
			ConstSFVec3f tmp_pendelCenter = (ConstSFVec3f)e.getValue();
			tmp_pendelCenter.getValue(tmpPendelBBox);
			
			SFNode myPendel = ((vrml.field.SFNode)getField("Pendel"));
			SFRotation pendelRotation = (SFRotation)((Node)myPendel.getValue()).getExposedField("rotation");
			
			float pendelRotationArray[] = new float[4];
			pendelRotation.getValue(pendelRotationArray);
			
			if( pendelRotationArray[3] < 0)
			{
				energyPoint[0] = tmpPendelBBoxCenter[0] - 0.5F * tmpPendelBBox[0] + 0.1F;
				energyPoint[1] = tmpPendelBBoxCenter[1] - 0.5F * tmpPendelBBox[1] + 0.1F;
				energyPoint[2] = tmpPendelBBoxCenter[2];
			} else {
				energyPoint[0] = tmpPendelBBoxCenter[0] + 0.5F * tmpPendelBBox[0] - 0.1F;
				energyPoint[1] = tmpPendelBBoxCenter[1] - 0.5F * tmpPendelBBox[1] + 0.1F;
				energyPoint[2] = tmpPendelBBoxCenter[2];			
			}
		}

		setTimer();
		draw();
    }

	
    public void evolveParticle()
    {
    	for(int i=0; i < realNumParticles; i++){      // evolve the particle parameters
    		
    		//TODO: animieren sie die partikel
    		
    		//TODO: farbwechsel und skalierung berechnen
    	}
    }
    
	
	public void draw() 
	{
		SFNode ParticlesIN = (SFNode)getField("ParticlesIN");
    	MFNode n1 = (MFNode)((Node)ParticlesIN.getValue()).getExposedField("children");
	
   		for (int i=0; i < realNumParticles; i++)
   		{
   			Particles tmpParticle = myParticleArray[i];
		
			float [] myPos = new float [3];
			float [] myCol = new float [3];
			
			myPos = tmpParticle.getPos();
			myCol = tmpParticle.getColor();
			
			vrml.field.SFVec3f set_translation = (SFVec3f)((Node)n1.get1Value(i)).getEventIn("set_translation");
			set_translation.setValue(myPos[0], myPos[1], myPos[2]);
   			
    		//TODO: farbwechsel und skalierung muss hier bergeben werden
		}
		evolveParticle();
	}

    /*! \brief convert deg to rad
     * 
     */
	private float rad( float deg)
	{
		return deg/360*(float)Math.PI;
	}

	/*! \brief convert rad to deg
	 * 
	 */
	private float deg( float rad)
	{
		return rad * 180 / (float)Math.PI;
	}

} // END CLASS